﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;
using System.Xml;

namespace Ultra_Starfield
{
    public partial class Form1 : Form
    {

        public static class Variables
        {
            public static int StarCount = 30000;
            public static double StarSizeIncreasePerStep = 0.00025; //0.00025 (or about 0.02 for interesting effect)
            public static double StarAlphaIncreasePerStep = 0.15; // Used to increase star alpha as it approaches, should be about 0.15 depending on speed
            public static int StarStartingAlpha = 10; // should be about 10 - 50
            public static double SpeedFactor = 0.5; // about 0.5
            public static double InitialStarSize = 1;
            public static double CometTailLength = 0.2; // about 0.2
            public static int CometTailWidth = 1;
            public static int StarColorIndex = 1;
            public static bool ColorfulStars = false;
            public static bool ExtraDim = true;
            public static bool DefaultMode = true;
            public static bool PartyMode = false;
            public static bool ClassicMode = false;
            public static bool StarTrekMode = true;
            public static bool SyncParameters = true;
            public static bool CometTails = false;
            public static bool CustomStarColor = false;
            public static bool SilenceErrors = true;
            public static byte CustomStarColorR = 255;
            public static byte CustomStarColorG = 255;
            public static byte CustomStarColorB = 255;
            public static byte PictureBoxBackgroundColorR = 0;
            public static byte PictureBoxBackgroundColorG = 0;
            public static byte PictureBoxBackgroundColorB = 0;
            public static int TimerInterval = 40;
        }

        private ArrayList stars = new ArrayList();
        List<double> StarCoordinateX_List = new List<double>();
        List<double> StarCoordinateY_List = new List<double>();
        List<double> starsZ = new List<double>();
        List<double> starsR = new List<double>();
        List<double> starsG = new List<double>();
        List<double> starsB = new List<double>();
        List<double> starsSize = new List<double>();
        List<double> starsSpeed = new List<double>();

        public double x;
        public double y;
        public double z;
        public int r;
        public int g;
        public int b;
        public double size = 1;
        public int CometTailEndPointX;
        public int CometTailEndPointY;
        public float StarDistanceFromCenterX;
        public float StarDistanceFromCenterY;
        public float ScreenCenterX;
        public float ScreenCenterY;
        private readonly Random rand = new Random();
        public int TimerCount = 0;

        public Form1()
        {
            try
            {
                InitializeComponent();
                this.timer1.Tick += new System.EventHandler(this.Timer1_Tick);
                timer1.Interval = Variables.TimerInterval;
                InitializeLists();
                PictureBox1.Image = new Bitmap(PictureBox1.Width, PictureBox1.Height);
                // Read and set all saved variables
                ReadVariables("ultra_starfield_settings.xml");
                PictureBox1.BackColor = Color.FromArgb(255, Variables.PictureBoxBackgroundColorR, Variables.PictureBoxBackgroundColorG, Variables.PictureBoxBackgroundColorB);
                // Label colors default to be visible during design time
                // Change colors to proper runtime colors
                LabelInstructions.BackColor = Color.Black;
                LabelInstructions.ForeColor = Color.Gray;
                Cursor.Hide();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void InitializeLists()
        {
            try
            {
                for (int i = 0; i < Variables.StarCount; i++)
                {
                    StarCoordinateX_List.Add(0);
                    StarCoordinateY_List.Add(0);
                    starsZ.Add(0);
                    starsR.Add(0);
                    starsG.Add(0);
                    starsB.Add(0);
                    starsSize.Add(1);
                    starsSpeed.Add(1); // will be replaced by random number
                }
                Variables.StarCount = 3000; // first initialize the lists with 30000, then reduce star count to 3000 to run better
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void Timer1_Tick(object sender, System.EventArgs e)
        {
            try
            {
                AdvanceStars();
                PictureBox1.Refresh();

                // Start hiding instruction label aftet timer count reaches 75 (about 3 seconds)
                // Instruction label should be gone at about timer count 100
                // Allow timer count to run up to 200 just in case
                // Timer interval of 40 = 25 counts / second
                if (TimerCount < 200)
                {
                    TimerCount = TimerCount + 1;
                }
                if (TimerCount > 75 && LabelInstructions.Height > 0)
                {
                    LabelInstructions.Height = LabelInstructions.Height - 1;
                }
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error in timer_Tick", MessageBoxButtons.OK);
            }
        }

        public void AdvanceStars()
        {
            // Move the stars 1 step, also increase alpha value and size
            try
            {
                for (int i = 0; i < Variables.StarCount; i++)
                {
                    StarCoordinateX_List[i] = StarCoordinateX_List[i] + (StarCoordinateX_List[i] - .5) * starsSpeed[i] * Variables.SpeedFactor;
                    StarCoordinateY_List[i] = StarCoordinateY_List[i] + (StarCoordinateY_List[i] - .5) * starsSpeed[i] * Variables.SpeedFactor;
                    starsZ[i] = starsZ[i] + Variables.StarAlphaIncreasePerStep * Math.Pow(starsSpeed[i] * 100, 2);
                    starsSize[i] = starsSize[i] + Variables.StarSizeIncreasePerStep * Math.Pow(starsSpeed[i] * 100, 2);

                    // Reset stars that went off the screen
                    // Allow the comet tails to continue off the screen
                    if (Variables.CometTails)
                    {
                        if (StarCoordinateX_List[i] < 0 - Variables.CometTailLength || StarCoordinateX_List[i] > 1 + Variables.CometTailLength || StarCoordinateY_List[i] < 0 - Variables.CometTailLength || StarCoordinateY_List[i] > 1 + Variables.CometTailLength)
                            MakeNewStar(i);
                    }
                    // No comet tails
                    else if (StarCoordinateX_List[i] < 0 || StarCoordinateX_List[i] > 1 || StarCoordinateY_List[i] < 0 || StarCoordinateY_List[i] > 1)
                        MakeNewStar(i);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error in AdvanceStars", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void MakeNewStar(int i)
        {
            // Create a new star
            // Move star color, position, size, and alpha values into lists
            try
            {
                // set color for all stars to white
                int r_temp = 255;
                int g_temp = 255;
                int b_temp = 255;

                double random_speed;

                if (Variables.ColorfulStars)
                {
                    if (Variables.StarColorIndex == 1) // red
                    {
                        r_temp = 255;
                        g_temp = 0;
                        b_temp = 0;
                        Variables.StarColorIndex++;
                    }
                    else if (Variables.StarColorIndex == 2) // orange
                    {
                        r_temp = 255;
                        g_temp = 128;
                        b_temp = 0;
                        Variables.StarColorIndex++;
                    }
                    else if (Variables.StarColorIndex == 3) // yellow
                    {
                        r_temp = 255;
                        g_temp = 255;
                        b_temp = 0;
                        Variables.StarColorIndex++;
                    }
                    else if (Variables.StarColorIndex == 4) // green
                    {
                        r_temp = 0;
                        g_temp = 255;
                        b_temp = 0;
                        Variables.StarColorIndex++;
                    }
                    else if (Variables.StarColorIndex == 5) // blue
                    {
                        r_temp = 0;
                        g_temp = 128;
                        b_temp = 255;
                        Variables.StarColorIndex++;
                    }
                    else if (Variables.StarColorIndex == 6) // purple
                    {
                        r_temp = 128;
                        g_temp = 0;
                        b_temp = 255;
                        Variables.StarColorIndex = 1;
                    }
                }
                else if (Variables.ClassicMode)
                {
                    // do nothing, keep white color
                }
                else if (Variables.CustomStarColor)
                {
                    r_temp = Variables.CustomStarColorR;
                    g_temp = Variables.CustomStarColorG;
                    b_temp = Variables.CustomStarColorB;
                }
                else // default mode
                {
                    if (i % 2 == 0) // only even numbered stars get random color
                    {
                        // set random star color (red green blue)
                        r_temp = Convert.ToInt32(rand.NextDouble() * (1 - 0.9) + 0.9) * 255; // close to 255
                        g_temp = Convert.ToInt32(rand.NextDouble() * (0.9 - 0.8) + 0.8) * 255; // 0 to 200
                        b_temp = Convert.ToInt32(rand.NextDouble() * (0.9 - 0.6) + 0.6) * 255; // close to 0
                    }
                }

                // set star color (red green blue)
                SetStarColor(i, r_temp, g_temp, b_temp);


                // Scale the position around 0.5 (center of screen) using * (maxNumber - minNumber) + minNumber
                // Closer to 0.5 will keep new stars appearing closer to the center of screen
                x = rand.NextDouble() * (0.7 - 0.3) + 0.3;
                y = rand.NextDouble() * (0.7 - 0.3) + 0.3;

                StarCoordinateX_List[i] = x;
                StarCoordinateY_List[i] = y;

                starsZ[i] = Variables.StarStartingAlpha;

                // Speed (step size), get random number between 1 - 6
                random_speed = rand.NextDouble() * (6 - 1) + 1;
                if (Variables.ClassicMode)
                {
                    random_speed = rand.NextDouble() * (2.5 - 1) + 1;
                }
                if (Variables.ExtraDim)
                {
                    starsSpeed[i] = random_speed * 0.01;
                }
                else
                {
                    starsSpeed[i] = 0.05;
                }

                // https://stackoverflow.com/questions/5294955/how-to-scale-down-a-range-of-numbers-with-a-known-min-and-max-value
                // Make starting star size relative to star speed
                // Scale star speed (1 - 6) down to 0.5 - 1.5
                //InitialStarSize = (maxAllowed - minAllowed) * (unscaledNum - min) / (max - min) + minAllowed;
                double InitialStarSizeScale = (1.4 - 0.5) * (random_speed - 1) / (6 - 1) + 0.5;
                starsSize[i] = Variables.InitialStarSize * InitialStarSizeScale; // 0.5 - 1.5
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error in MakeNewStar", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void SetStarColor(int i, int r, int g, int b)
        {
            try
            {
                starsR[i] = r;
                starsG[i] = g;
                starsB[i] = b;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                // Dock the PictureBox to the form and set its background to black.
                PictureBox1.Dock = DockStyle.Fill;

                // Add the PictureBox control to the Form.
                this.Controls.Add(PictureBox1);
                this.WindowState = FormWindowState.Normal;
                this.TopMost = true;
                this.FormBorderStyle = FormBorderStyle.None;
                this.WindowState = FormWindowState.Maximized;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // Points used for comet tail start and end points
        // Create outside of the paint event to prevent new point objects being continiously created
        static Point CometTailStartPoint = new Point(1, 10);
        static Point CometTailEndPoint = new Point(3, 30);
        private void PictureBox1_Paint(object sender, PaintEventArgs e)
        {
            try
            {
                e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

                ScreenCenterX = (float)(this.Width * 0.5);
                ScreenCenterY = (float)(this.Height * 0.5);

                for (int i = 0; i < Variables.StarCount; i++)
                {
                    float StarCoordinateX = (float)StarCoordinateX_List[i] * this.Width;
                    float StarCoordinateY = (float)StarCoordinateY_List[i] * this.Height;

                    float radius = (float)starsSize[i];
                    float diameter = radius * 2;

                    // Limit alpha to 255
                    if (starsZ[i] >= 255)
                    {
                        starsZ[i] = 255;
                    }

                    // Paint the stars
                    System.Drawing.Color? starColor = Color.FromArgb((int)starsZ[i], (int)starsR[i], (int)starsG[i], (int)starsB[i]);
                    using (var brush = new SolidBrush(starColor.Value))
                        e.Graphics.FillEllipse(brush, StarCoordinateX - radius, StarCoordinateY - radius, diameter, diameter);
                    
                    // Make the comet tails
                    if (Variables.CometTails)
                    {
                        GetStarDistanceFromCenter(StarCoordinateX, StarCoordinateY, ScreenCenterX, ScreenCenterY);

                        float CometTailLengthX = (float)(StarDistanceFromCenterX * Variables.CometTailLength);
                        float CometTailLengthY = (float)(StarDistanceFromCenterY * Variables.CometTailLength);

                        GetCometTailEndPoint(StarCoordinateX, StarCoordinateY, ScreenCenterX, ScreenCenterY, CometTailLengthX, CometTailLengthY);

                        CometTailStartPoint.X = (int)StarCoordinateX;
                        CometTailStartPoint.Y = (int)StarCoordinateY;
                        CometTailEndPoint.X = CometTailEndPointX;
                        CometTailEndPoint.Y = CometTailEndPointY;

                        // check for valid brush parameters
                        // bad parameters cause out of memory errors
                        if (CometTailStartPoint != CometTailEndPoint)
                        {
                            // Star Trek mode draws 2 lines
                            // 1st line is normal, 2nd line trails behind and is connected
                            if (Variables.StarTrekMode)
                            { 
                                LinearGradientBrush myLinearGradientBrush = new LinearGradientBrush(
                                CometTailStartPoint,
                                CometTailEndPoint,
                                Color.FromArgb((int)starsZ[i], 255, 32, 32), // Red, leading edge of comet trail
                                Color.FromArgb((int)starsZ[i], 255, 255, 0)); // yellow
                                Pen pen = new Pen(myLinearGradientBrush, Variables.CometTailWidth);
                                e.Graphics.DrawLine(pen, (int)StarCoordinateX, (int)StarCoordinateY, CometTailEndPointX, CometTailEndPointY);

                                // Make the end points the new start points for 2nd line
                                float CometTailStartPointX = CometTailEndPointX;
                                float CometTailStartPointY = CometTailEndPointY;

                                // Position the 2nd line behind the 1st
                                CometTailLengthX = (float)(StarDistanceFromCenterX * Variables.CometTailLength * 2);
                                CometTailLengthY = (float)(StarDistanceFromCenterY * Variables.CometTailLength * 2);

                                GetCometTailEndPoint(StarCoordinateX, StarCoordinateY, ScreenCenterX, ScreenCenterY, CometTailLengthX, CometTailLengthY);

                                CometTailEndPoint.X = CometTailEndPointX;
                                CometTailEndPoint.Y = CometTailEndPointY;

                                if (CometTailStartPoint != CometTailEndPoint)
                                {
                                    // 2nd line
                                    LinearGradientBrush myLinearGradientBrush2 = new LinearGradientBrush(
                                    CometTailStartPoint,
                                    CometTailEndPoint,
                                    Color.FromArgb((int)starsZ[i], 64, 255, 128), // White / green
                                    Color.FromArgb((int)starsZ[i], 0, 64, 255)); // Blue, end of comet tail

                                    Pen pen2 = new Pen(myLinearGradientBrush2, Variables.CometTailWidth);
                                    e.Graphics.DrawLine(pen2, CometTailStartPointX, CometTailStartPointY, CometTailEndPointX, CometTailEndPointY);
                                }
                            }
                            else // Not Star Trek
                            {
                                LinearGradientBrush myLinearGradientBrush = new LinearGradientBrush(
                                CometTailStartPoint,
                                CometTailEndPoint,
                                Color.FromArgb((int)starsZ[i], (int)starsR[i], (int)starsG[i], (int)starsB[i]),
                                Color.FromArgb((int)(starsZ[i] * 0.8), (int)(starsR[i] * 0.1), (int)(starsG[i] * 0.1), (int)(starsB[i] * 0.1)));
                                Pen pen = new Pen(myLinearGradientBrush, Variables.CometTailWidth);
                                e.Graphics.DrawLine(pen, (int)StarCoordinateX, (int)StarCoordinateY, CometTailEndPointX, CometTailEndPointY);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error in PictureBox1_Paint", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void PictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            try
            {
                if (e.Button == MouseButtons.Right)
                {
                    //var frm = new frmOptions();
                    // pass a reference to the timer in the second form's constructor
                    // thus enabling the second form to manipulate any property of the timer
                    frmOptions frm = new frmOptions(this.timer1, this.PictureBox1);
                    //frmOptions frm = new frmOptions(this.PictureBox1);

                    frm.TopMost = true;
                    frm.Show();
                }
                else//left or middle click
                {
                    WriteVariables("ultra_starfield_settings.xml");
                    System.Windows.Forms.Application.Exit();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            try
            {
                //capture up arrow key
                if (keyData == Keys.Up)
                {
                    if (Variables.SpeedFactor < 30)
                    {
                        Variables.SpeedFactor = Variables.SpeedFactor + 0.02;
                        Variables.StarAlphaIncreasePerStep = (Variables.SpeedFactor * 2) / 3.33333;
                        Variables.StarSizeIncreasePerStep = (Variables.SpeedFactor * 2) / 2000;
                    }
                    return true;
                }
                //capture down arrow key
                if (keyData == Keys.Down)
                {
                    if (Variables.SpeedFactor > 0.05)
                    {
                        Variables.SpeedFactor = Variables.SpeedFactor - 0.02;
                        Variables.StarAlphaIncreasePerStep = (Variables.SpeedFactor * 2) / 3.333333;
                        Variables.StarSizeIncreasePerStep = (Variables.SpeedFactor * 2) / 2000;
                    }
                    return true;
                }
                //capture left arrow key
                if (keyData == Keys.Left)
                {
                    if (Variables.StarCount > 300)
                    {
                        Variables.StarCount = Variables.StarCount - 20;
                    }
                    return true;
                }
                //capture right arrow key
                if (keyData == Keys.Right)
                {
                    if (Variables.StarCount <= 29980)
                    {
                        Variables.StarCount = Variables.StarCount + 20;
                    }
                    return true;
                }
                //capture ctrl+o
                if (keyData == (Keys.Control | Keys.O))
                {
                    // open options form
                    frmOptions frm = new frmOptions(this.timer1, this.PictureBox1);
                    return true;
                }
                WriteVariables("ultra_starfield_settings.xml");
                System.Windows.Forms.Application.Exit();
                return base.ProcessCmdKey(ref msg, keyData);
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return true;
            }
        }

        public static void WriteVariables(string FilePath)
        {
            try
            {
                XmlWriterSettings writerSettings = new XmlWriterSettings
                {
                    Indent = true,
                    OmitXmlDeclaration = true
                };
                using XmlWriter xmlWriter = XmlWriter.Create(FilePath, writerSettings);
                xmlWriter.WriteStartDocument();
                xmlWriter.WriteStartElement("Ultra_Starfield");
                xmlWriter.WriteAttributeString("ver", "1");

                xmlWriter.WriteStartElement("StarCount");
                xmlWriter.WriteString(Variables.StarCount.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("StarSizeIncreasePerStep");
                xmlWriter.WriteString(Variables.StarSizeIncreasePerStep.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("StarAlphaIncreasePerStep");
                xmlWriter.WriteString(Variables.StarAlphaIncreasePerStep.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("StarStartingAlpha");
                xmlWriter.WriteString(Variables.StarStartingAlpha.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("SpeedFactor");
                xmlWriter.WriteString(Variables.SpeedFactor.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("InitialStarSize");
                xmlWriter.WriteString(Variables.InitialStarSize.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("ColorfulStars");
                xmlWriter.WriteString(Variables.ColorfulStars.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("ExtraDim");
                xmlWriter.WriteString(Variables.ExtraDim.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("DefaultMode");
                xmlWriter.WriteString(Variables.DefaultMode.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("PartyMode");
                xmlWriter.WriteString(Variables.PartyMode.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("ClassicMode");
                xmlWriter.WriteString(Variables.ClassicMode.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("StarTrekMode");
                xmlWriter.WriteString(Variables.StarTrekMode.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("SyncParameters");
                xmlWriter.WriteString(Variables.SyncParameters.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("CometTails");
                xmlWriter.WriteString(Variables.CometTails.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("CometTailLength");
                xmlWriter.WriteString(Variables.CometTailLength.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("CometTailWidth");
                xmlWriter.WriteString(Variables.CometTailWidth.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("TimerInterval");
                xmlWriter.WriteString(Variables.TimerInterval.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("CustomStarColor");
                xmlWriter.WriteString(Variables.CustomStarColor.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("CustomStarColorR");
                xmlWriter.WriteString(Variables.CustomStarColorR.ToString());
                xmlWriter.WriteEndElement();
                xmlWriter.WriteStartElement("CustomStarColorG");
                xmlWriter.WriteString(Variables.CustomStarColorG.ToString());
                xmlWriter.WriteEndElement();
                xmlWriter.WriteStartElement("CustomStarColorB");
                xmlWriter.WriteString(Variables.CustomStarColorB.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("PictureBoxBackgroundColorR");
                xmlWriter.WriteString(Variables.PictureBoxBackgroundColorR.ToString());
                xmlWriter.WriteEndElement();
                xmlWriter.WriteStartElement("PictureBoxBackgroundColorG");
                xmlWriter.WriteString(Variables.PictureBoxBackgroundColorG.ToString());
                xmlWriter.WriteEndElement();
                xmlWriter.WriteStartElement("PictureBoxBackgroundColorB");
                xmlWriter.WriteString(Variables.PictureBoxBackgroundColorB.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteStartElement("SilenceErrors");
                xmlWriter.WriteString(Variables.SilenceErrors.ToString());
                xmlWriter.WriteEndElement();

                xmlWriter.WriteEndDocument();
                xmlWriter.Flush();
                xmlWriter.Close();
            }
            catch (Exception ex)
            {
                // disable for screensaver usage
                if (!Variables.SilenceErrors)
                {
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        public static void ReadVariables(string FilePath)
        {
            try
            {
                // make sure file exists
                if (!File.Exists(FilePath))
                {
                    return;
                }

                using XmlReader txtReader = new XmlTextReader(FilePath);
                //* create an xml document object.
                XmlDocument xmlDoc = new XmlDocument();

                try
                {
                    xmlDoc.Load(txtReader); //* load the XML document from the specified file.
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Sorry, I could not open the file: " + FilePath + "\r\n" + ex.Message, "Open Error", MessageBoxButtons.OK);
                    return;
                }

                //test for correct "teslamap" root attribute (version)
                string ver = "";

                try
                {
                    ver = xmlDoc.SelectSingleNode("/Ultra_Starfield/@ver").Value;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Sorry, I could not open the file: " + FilePath + "\r\n" + "Incorrect file format." + "\r\n" + ex.Message, "Open Error", MessageBoxButtons.OK);
                    return;
                }

                if (ver != "1")
                {
                    MessageBox.Show("Sorry, I could not open the file: " + FilePath + "\r\n" + "Version incompatibility.", "Open Error", MessageBoxButtons.OK);
                    return;
                }

                // get info from the XML file

                XmlNodeList StarCount = xmlDoc.GetElementsByTagName("StarCount");
                Variables.StarCount = Convert.ToInt32(StarCount[0].InnerText);

                XmlNodeList StarSizeIncreasePerStep = xmlDoc.GetElementsByTagName("StarSizeIncreasePerStep");
                Variables.StarSizeIncreasePerStep = Convert.ToDouble(StarSizeIncreasePerStep[0].InnerText);

                XmlNodeList StarAlphaIncreasePerStep = xmlDoc.GetElementsByTagName("StarAlphaIncreasePerStep");
                Variables.StarAlphaIncreasePerStep = Convert.ToDouble(StarAlphaIncreasePerStep[0].InnerText);

                XmlNodeList StarStartingAlpha = xmlDoc.GetElementsByTagName("StarStartingAlpha");
                Variables.StarStartingAlpha = Convert.ToInt32(StarStartingAlpha[0].InnerText);

                XmlNodeList SpeedFactor = xmlDoc.GetElementsByTagName("SpeedFactor");
                Variables.SpeedFactor = Convert.ToDouble(SpeedFactor[0].InnerText);

                XmlNodeList InitialStarSize = xmlDoc.GetElementsByTagName("InitialStarSize");
                Variables.InitialStarSize = Convert.ToDouble(InitialStarSize[0].InnerText);

                XmlNodeList ColorfulStars = xmlDoc.GetElementsByTagName("ColorfulStars");
                Variables.ColorfulStars = bool.Parse(ColorfulStars[0].InnerText);

                XmlNodeList ExtraDim = xmlDoc.GetElementsByTagName("ExtraDim");
                Variables.ExtraDim = bool.Parse(ExtraDim[0].InnerText);

                XmlNodeList DefaultMode = xmlDoc.GetElementsByTagName("DefaultMode");
                Variables.DefaultMode = bool.Parse(DefaultMode[0].InnerText);

                XmlNodeList PartyMode = xmlDoc.GetElementsByTagName("PartyMode");
                Variables.PartyMode = bool.Parse(PartyMode[0].InnerText);

                XmlNodeList ClassicMode = xmlDoc.GetElementsByTagName("ClassicMode");
                Variables.ClassicMode = bool.Parse(ClassicMode[0].InnerText);

                XmlNodeList StarTrekMode = xmlDoc.GetElementsByTagName("StarTrekMode");
                Variables.StarTrekMode = bool.Parse(StarTrekMode[0].InnerText);

                XmlNodeList SyncParameters = xmlDoc.GetElementsByTagName("SyncParameters");
                Variables.SyncParameters = bool.Parse(SyncParameters[0].InnerText);

                XmlNodeList CometTails = xmlDoc.GetElementsByTagName("CometTails");
                Variables.CometTails = bool.Parse(CometTails[0].InnerText);

                XmlNodeList CometTailLength = xmlDoc.GetElementsByTagName("CometTailLength");
                Variables.CometTailLength = Convert.ToDouble(CometTailLength[0].InnerText);

                XmlNodeList CometTailWidth = xmlDoc.GetElementsByTagName("CometTailWidth");
                Variables.CometTailWidth = Convert.ToInt32(CometTailWidth[0].InnerText);

                XmlNodeList TimerInterval = xmlDoc.GetElementsByTagName("TimerInterval");
                Variables.TimerInterval = Convert.ToInt32(TimerInterval[0].InnerText);

                XmlNodeList CustomStarColor = xmlDoc.GetElementsByTagName("CustomStarColor");
                Variables.CustomStarColor = bool.Parse(CustomStarColor[0].InnerText);

                XmlNodeList CustomStarColorR = xmlDoc.GetElementsByTagName("CustomStarColorR");
                Variables.CustomStarColorR = Byte.Parse(CustomStarColorR[0].InnerText);

                XmlNodeList CustomStarColorG = xmlDoc.GetElementsByTagName("CustomStarColorG");
                Variables.CustomStarColorG = Byte.Parse(CustomStarColorG[0].InnerText);

                XmlNodeList CustomStarColorB = xmlDoc.GetElementsByTagName("CustomStarColorB");
                Variables.CustomStarColorB = Byte.Parse(CustomStarColorB[0].InnerText);

                XmlNodeList PictureBoxBackgroundColorR = xmlDoc.GetElementsByTagName("PictureBoxBackgroundColorR");
                Variables.PictureBoxBackgroundColorR = Byte.Parse(PictureBoxBackgroundColorR[0].InnerText);

                XmlNodeList PictureBoxBackgroundColorG = xmlDoc.GetElementsByTagName("PictureBoxBackgroundColorG");
                Variables.PictureBoxBackgroundColorG = Byte.Parse(PictureBoxBackgroundColorG[0].InnerText);

                XmlNodeList PictureBoxBackgroundColorB = xmlDoc.GetElementsByTagName("PictureBoxBackgroundColorB");
                Variables.PictureBoxBackgroundColorB = Byte.Parse(PictureBoxBackgroundColorB[0].InnerText);

                XmlNodeList SilenceErrors = xmlDoc.GetElementsByTagName("SilenceErrors");
                Variables.SilenceErrors = bool.Parse(SilenceErrors[0].InnerText);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error in ReadVariables", MessageBoxButtons.OK);
            }
        }

        public void GetCometTailEndPoint(float StarCoordinateXF, float StarCoordinateYF, float ScreenCenterXF, float ScreenCenterYF, float CometTailLengthXF, float CometTailLengthYF)
        {
            // top left quadrant ++
            if (StarCoordinateXF <= ScreenCenterXF && StarCoordinateYF <= ScreenCenterYF)
            {
                CometTailEndPointX = (int)(StarCoordinateXF + CometTailLengthXF);
                CometTailEndPointY = (int)(StarCoordinateYF + CometTailLengthYF);
            }

            // top right quadrant -+
            else if (StarCoordinateXF >= ScreenCenterXF && StarCoordinateYF <= ScreenCenterYF)
            {
                CometTailEndPointX = (int)(StarCoordinateXF - CometTailLengthXF);
                CometTailEndPointY = (int)(StarCoordinateYF + CometTailLengthYF);
            }

            // bottom right quadrant --
            else if (StarCoordinateXF >= ScreenCenterXF && StarCoordinateYF >= ScreenCenterYF)
            {
                CometTailEndPointX = (int)(StarCoordinateXF - CometTailLengthXF);
                CometTailEndPointY = (int)(StarCoordinateYF - CometTailLengthYF);
            }

            // bottom left quadrant +-
            else
            {
                CometTailEndPointX = (int)(StarCoordinateXF + CometTailLengthXF);
                CometTailEndPointY = (int)(StarCoordinateYF - CometTailLengthYF);
            }
        }

        public void GetStarDistanceFromCenter(float StarCoordinateXf, float StarCoordinateYf, float ScreenCenterXf, float ScreenCenterYf)
        {
            StarDistanceFromCenterX = Math.Abs(StarCoordinateXf - ScreenCenterXf);
            StarDistanceFromCenterY = Math.Abs(StarCoordinateYf - ScreenCenterYf);
        }
    }
}
