﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

//scale range to new range...
//(New_Max - New_Min) * (RawValue - Original_min) / (Original_max - Original_min) + New_Min
//(Scaled_Max - Scaled_Min) * (RawValue - unscaled_min) / (unscaled_max - unscaled_min) + Scaled_Min

namespace Ultra_Starfield
{
    public partial class frmOptions : Form
    {
        private Timer TimerFromForm1;
        private PictureBox PictureBoxFromForm1;

        // pass a reference to the timer in the form's constructor
        public frmOptions(Timer ownerTimer, PictureBox ownerPictureBox) 
        {
            try
            {
                Cursor.Show();
                PictureBox pb1 = new PictureBox();
                PictureBoxFromForm1 = ownerPictureBox;

                InitializeComponent();
                this.MaximizeBox = false;
                this.MinimizeBox = false;
                SetSliders();
                SetButtons();
                GroupBoxMasterClockFrequency.BackColor = Color.FromArgb(255, 40, 40, 40);
                TimerFromForm1 = ownerTimer;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        private static void SyncParameters(double SpeedFactorF, double StarAlphaIncreasePerStepF, double StarSizeIncreasePerStepF)
        {
            // Maintain proportions between variables to keep the starfield looking proper
            try
            {
                if (SpeedFactorF != 0)
                {
                    Form1.Variables.StarAlphaIncreasePerStep = SpeedFactorF / 3.3333;
                    Form1.Variables.StarSizeIncreasePerStep = SpeedFactorF / 2000;
                    return;
                }

                if (StarAlphaIncreasePerStepF != 0)
                {
                    Form1.Variables.SpeedFactor = StarAlphaIncreasePerStepF * 3.3333;
                    Form1.Variables.StarSizeIncreasePerStep = StarAlphaIncreasePerStepF * 0.0016667;
                    return;
                }

                if (StarSizeIncreasePerStepF != 0)
                {
                    Form1.Variables.SpeedFactor = StarSizeIncreasePerStepF * 2000;
                    Form1.Variables.StarAlphaIncreasePerStep = StarSizeIncreasePerStepF * 600;
                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void SetSliders()
        {
            try
            {
                // Take the scaled variable value and reverse scale it logarithmicly, assign the result to trackbar value
                TrackBarStars.Value = SetLogarithmicSlider(Form1.Variables.StarCount, 300, 30000);
                labelStars.Text = Convert.ToInt32(Form1.Variables.StarCount).ToString();

                TrackBarVelocity.Value = SetLogarithmicSlider(Form1.Variables.SpeedFactor, 0.05, 30);
                labelVelocity.Text = (TrackBarVelocity.Value + 1).ToString();

                TrackBarStarSize.Value = SetLogarithmicSlider(Form1.Variables.InitialStarSize, 0.05, 30);
                labelStarSize.Text = (TrackBarStarSize.Value + 1).ToString();

                TrackBarStarSizeIncreaseRate.Value = SetLogarithmicSlider(Form1.Variables.StarSizeIncreasePerStep, 0.00001, 0.1);
                labelStarSizeIncreaseRate.Text = (TrackBarStarSizeIncreaseRate.Value + 1).ToString();

                TrackBarBrightnessIncreaseRate.Value = SetLogarithmicSlider(Form1.Variables.StarAlphaIncreasePerStep, 0.01, 10);
                labelBrightnessIncreaseRate.Text = (TrackBarBrightnessIncreaseRate.Value + 1).ToString();

                TrackBarCometTailLength.Value = (int)(Form1.Variables.CometTailLength * 10);
                labelCometTailLength.Text = Convert.ToInt32((Form1.Variables.CometTailLength * 10)).ToString();

                TrackBarCometTailWidth.Value = (int)(Form1.Variables.CometTailWidth);
                labelCometTailWidth.Text = (Form1.Variables.CometTailWidth).ToString();

                int x = Form1.Variables.TimerInterval / 10;
                TrackBarMasterClockFrequency.Value = ((11 + 1) - x) - 1; // Invert interval to get trackbar value
                LabelMasterClockFrequency.Text = (Form1.Variables.TimerInterval).ToString() + " ms";

                if (Form1.Variables.StarTrekMode || Form1.Variables.CometTails)
                {
                    TrackBarCometTailLength.Enabled = true;
                    TrackBarCometTailWidth.Enabled = true;
                }
                else
                {
                    TrackBarCometTailLength.Enabled = false;
                    TrackBarCometTailWidth.Enabled = false;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void SetButtons()
        {
            try
            {
                if (!Form1.Variables.SyncParameters)
                {
                    ButtonSyncParameters.Font = new Font(ButtonSyncParameters.Font, FontStyle.Bold);
                    ButtonSyncParameters.Text = "OFF";
                    ButtonSyncParameters.BackColor = Color.Green;
                }
                else
                {
                    ButtonSyncParameters.Font = new Font(ButtonSyncParameters.Font, FontStyle.Regular);
                    ButtonSyncParameters.Text = "ON";
                    ButtonSyncParameters.BackColor = Color.LawnGreen;
                }

                if (!Form1.Variables.ExtraDim)
                {
                    ButtonUltraDepth.Font = new Font(ButtonUltraDepth.Font, FontStyle.Bold);
                    ButtonUltraDepth.Text = "OFF";
                    ButtonUltraDepth.BackColor = Color.Green;
                }
                else
                {
                    ButtonUltraDepth.Font = new Font(ButtonUltraDepth.Font, FontStyle.Regular);
                    ButtonUltraDepth.Text = "ON";
                    ButtonUltraDepth.BackColor = Color.LawnGreen;
                }

                if (!Form1.Variables.ColorfulStars)
                {
                    ButtonUltraColor.Font = new Font(ButtonUltraColor.Font, FontStyle.Bold);
                    ButtonUltraColor.Text = "OFF";
                    ButtonUltraColor.BackColor = Color.Green;
                }
                else
                {
                    ButtonUltraColor.Font = new Font(ButtonUltraColor.Font, FontStyle.Regular);
                    ButtonUltraColor.Text = "ON";
                    ButtonUltraColor.BackColor = Color.LawnGreen;
                }

                if (!Form1.Variables.CometTails)
                {
                    ButtonCometTails.Font = new Font(ButtonCometTails.Font, FontStyle.Bold);
                    ButtonCometTails.Text = "OFF";
                    ButtonCometTails.BackColor = Color.Green;
                }
                else
                {
                    ButtonCometTails.Font = new Font(ButtonCometTails.Font, FontStyle.Regular);
                    ButtonCometTails.Text = "ON";
                    ButtonCometTails.BackColor = Color.LawnGreen;
                }

                if (!Form1.Variables.SilenceErrors)
                {
                    ButtonSilenceErrors.Font = new Font(ButtonSilenceErrors.Font, FontStyle.Bold);
                    ButtonSilenceErrors.Text = "OFF";
                    ButtonSilenceErrors.BackColor = Color.Green;
                }
                else
                {
                    ButtonSilenceErrors.Font = new Font(ButtonSilenceErrors.Font, FontStyle.Regular);
                    ButtonSilenceErrors.Text = "ON";
                    ButtonSilenceErrors.BackColor = Color.LawnGreen;
                }

                // Only one mode can be active at a time
                // Find what mode is active, then disable all others
                if (Form1.Variables.DefaultMode)
                {
                    Form1.Variables.DefaultMode = true;
                    ButtonDefaultMode.Font = new Font(ButtonDefaultMode.Font, FontStyle.Regular);
                    ButtonDefaultMode.Text = "ON";
                    ButtonDefaultMode.BackColor = Color.Yellow;

                    Form1.Variables.PartyMode = false;
                    ButtonPartyMode.Font = new Font(ButtonPartyMode.Font, FontStyle.Bold);
                    ButtonPartyMode.Text = "OFF";
                    ButtonPartyMode.BackColor = Color.Olive;

                    Form1.Variables.ClassicMode = false;
                    ButtonClassicMode.Font = new Font(ButtonClassicMode.Font, FontStyle.Bold);
                    ButtonClassicMode.Text = "OFF";
                    ButtonClassicMode.BackColor = Color.Olive;

                    Form1.Variables.StarTrekMode = false;
                    ButtonStarTrekMode.Font = new Font(ButtonStarTrekMode.Font, FontStyle.Bold);
                    ButtonStarTrekMode.Text = "OFF";
                    ButtonStarTrekMode.BackColor = Color.Olive;
                }
                else if (Form1.Variables.PartyMode)
                {
                    Form1.Variables.DefaultMode = false;
                    ButtonDefaultMode.Font = new Font(ButtonDefaultMode.Font, FontStyle.Bold);
                    ButtonDefaultMode.Text = "OFF";
                    ButtonDefaultMode.BackColor = Color.Olive;

                    Form1.Variables.PartyMode = true;
                    ButtonPartyMode.Font = new Font(ButtonPartyMode.Font, FontStyle.Regular);
                    ButtonPartyMode.Text = "ON";
                    ButtonPartyMode.BackColor = Color.Yellow;

                    Form1.Variables.ClassicMode = false;
                    ButtonClassicMode.Font = new Font(ButtonClassicMode.Font, FontStyle.Bold);
                    ButtonClassicMode.Text = "OFF";
                    ButtonClassicMode.BackColor = Color.Olive;

                    Form1.Variables.StarTrekMode = false;
                    ButtonStarTrekMode.Font = new Font(ButtonStarTrekMode.Font, FontStyle.Bold);
                    ButtonStarTrekMode.Text = "OFF";
                    ButtonStarTrekMode.BackColor = Color.Olive;
                }
                else if (Form1.Variables.ClassicMode)
                {
                    Form1.Variables.DefaultMode = false;
                    ButtonDefaultMode.Font = new Font(ButtonDefaultMode.Font, FontStyle.Bold);
                    ButtonDefaultMode.Text = "OFF";
                    ButtonDefaultMode.BackColor = Color.Olive;

                    Form1.Variables.PartyMode = false;
                    ButtonPartyMode.Font = new Font(ButtonPartyMode.Font, FontStyle.Bold);
                    ButtonPartyMode.Text = "OFF";
                    ButtonPartyMode.BackColor = Color.Olive;

                    Form1.Variables.ClassicMode = true;
                    ButtonClassicMode.Font = new Font(ButtonClassicMode.Font, FontStyle.Regular);
                    ButtonClassicMode.Text = "ON";
                    ButtonClassicMode.BackColor = Color.Yellow;

                    Form1.Variables.StarTrekMode = false;
                    ButtonStarTrekMode.Font = new Font(ButtonStarTrekMode.Font, FontStyle.Bold);
                    ButtonStarTrekMode.Text = "OFF";
                    ButtonStarTrekMode.BackColor = Color.Olive;
                }
                else // star trek mode
                {
                    Form1.Variables.DefaultMode = false;
                    ButtonDefaultMode.Font = new Font(ButtonDefaultMode.Font, FontStyle.Bold);
                    ButtonDefaultMode.Text = "OFF";
                    ButtonDefaultMode.BackColor = Color.Olive;

                    Form1.Variables.PartyMode = false;
                    ButtonPartyMode.Font = new Font(ButtonPartyMode.Font, FontStyle.Bold);
                    ButtonPartyMode.Text = "OFF";
                    ButtonPartyMode.BackColor = Color.Olive;

                    Form1.Variables.ClassicMode = false;
                    ButtonClassicMode.Font = new Font(ButtonClassicMode.Font, FontStyle.Bold);
                    ButtonClassicMode.Text = "OFF";
                    ButtonClassicMode.BackColor = Color.Olive;

                    Form1.Variables.StarTrekMode = true;
                    ButtonStarTrekMode.Font = new Font(ButtonStarTrekMode.Font, FontStyle.Regular);
                    ButtonStarTrekMode.Text = "ON";
                    ButtonStarTrekMode.BackColor = Color.Yellow;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        
        private static double ScaleLogarithmicSlider(int SliderValue, double OutputMin, double OutputMax)
        {
            // Take the trackbar value and scale it logarithmicly, assign the result to variable
            double x = 0.0;
            try
            {
                int InputMin = 0;
                int InputMax = 10;
                double OutputMinLog = Math.Log(OutputMin);
                double OutputMaxLog = Math.Log(OutputMax);
                double scale = (OutputMaxLog - OutputMinLog) / (InputMax - InputMin);
                x = Math.Exp(OutputMinLog + scale * (SliderValue - InputMin));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return x;
        }

        private static int SetLogarithmicSlider(double ScaledValue, double ScaledMin, double ScaledMax)
        {
            // Take the scaled variable value and reverse scale it logarithmicly, assign the result to trackbar value
            int RoundedX = 0;
            try
            {
                int InputMin = 0;
                int InputMax = 10;
                double OutputMinLog = Math.Log(ScaledMin);
                double OutputMaxLog = Math.Log(ScaledMax);
                double scale = (OutputMaxLog - OutputMinLog) / (InputMax - InputMin);
                double x = InputMin + (Math.Log(ScaledValue) - OutputMinLog) / scale;
                RoundedX = Convert.ToInt32(x);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return RoundedX;
        }

        private void ButtonSyncParameters_Click_1(object sender, EventArgs e)
        {
            try
            {
                if (Form1.Variables.SyncParameters)
                {
                    Form1.Variables.SyncParameters = false;
                }
                else
                {
                    Form1.Variables.SyncParameters = true;
                }
                SetButtons();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonUltraDepth_Click_1(object sender, EventArgs e)
        {
            try
            {
                if (Form1.Variables.ExtraDim)
                {
                    Form1.Variables.ExtraDim = false;
                }
                else
                {
                    Form1.Variables.ExtraDim = true;
                }
                SetButtons();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonUltraColor_Click_1(object sender, EventArgs e)
        {
            try
            {
                if (Form1.Variables.ColorfulStars)
                {
                    Form1.Variables.ColorfulStars = false;
                }
                else
                {
                    Form1.Variables.ColorfulStars = true;
                    Form1.Variables.CustomStarColor = false;
                }
                SetButtons();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCometTails_Click_1(object sender, EventArgs e)
        {
            try
            {
                if (Form1.Variables.CometTails)
                {
                    Form1.Variables.CometTails = false;
                    TrackBarCometTailLength.Enabled = false;
                    TrackBarCometTailWidth.Enabled = false;
                }
                else
                {
                    Form1.Variables.CometTails = true;
                    TrackBarCometTailLength.Enabled = true;
                    TrackBarCometTailWidth.Enabled = true;
                }
                SetButtons();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonDefaultMode_Click(object sender, EventArgs e)
        {
            try
            {
                if (!Form1.Variables.DefaultMode)
                {
                    Form1.Variables.DefaultMode = true;
                    Form1.Variables.PartyMode = false;
                    Form1.Variables.ClassicMode = false;
                    Form1.Variables.StarTrekMode = false;
                    Form1.Variables.CometTails = false;
                    Form1.Variables.ColorfulStars = false;
                    Form1.Variables.CustomStarColor = false;
                    Form1.Variables.StarCount = 3000;
                    Form1.Variables.StarSizeIncreasePerStep = 0.00025;
                    Form1.Variables.StarAlphaIncreasePerStep = 0.15;
                    Form1.Variables.StarStartingAlpha = 10;
                    Form1.Variables.SpeedFactor = 0.5;
                    Form1.Variables.InitialStarSize = 1;
                    Form1.Variables.PictureBoxBackgroundColorR = 0;
                    Form1.Variables.PictureBoxBackgroundColorG = 0;
                    Form1.Variables.PictureBoxBackgroundColorB = 0;
                    PictureBoxFromForm1.BackColor = Color.FromArgb(255, Form1.Variables.PictureBoxBackgroundColorR, Form1.Variables.PictureBoxBackgroundColorG, Form1.Variables.PictureBoxBackgroundColorB);
                    SetButtons();
                    SetSliders();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonPartyMode_Click(object sender, EventArgs e)
        {
            try
            {
                if (!Form1.Variables.PartyMode)
                {
                    Form1.Variables.DefaultMode = false;
                    Form1.Variables.PartyMode = true;
                    Form1.Variables.ClassicMode = false;
                    Form1.Variables.StarTrekMode = false;
                    Form1.Variables.CometTails = false;
                    Form1.Variables.ColorfulStars = true;
                    Form1.Variables.CustomStarColor = false;
                    Form1.Variables.StarCount = 800;
                    Form1.Variables.StarSizeIncreasePerStep = 0.1;
                    Form1.Variables.StarAlphaIncreasePerStep = 0.1;
                    Form1.Variables.StarStartingAlpha = 10;
                    Form1.Variables.SpeedFactor = 0.5;
                    Form1.Variables.InitialStarSize = 1;
                    Form1.Variables.PictureBoxBackgroundColorR = 0;
                    Form1.Variables.PictureBoxBackgroundColorG = 0;
                    Form1.Variables.PictureBoxBackgroundColorB = 0;
                    PictureBoxFromForm1.BackColor = Color.FromArgb(255, Form1.Variables.PictureBoxBackgroundColorR, Form1.Variables.PictureBoxBackgroundColorG, Form1.Variables.PictureBoxBackgroundColorB);
                    SetButtons();
                    SetSliders();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonClassicMode_Click(object sender, EventArgs e)
        {
            try
            {
                if (!Form1.Variables.ClassicMode)
                {
                    Form1.Variables.DefaultMode = false;
                    Form1.Variables.PartyMode = false;
                    Form1.Variables.ClassicMode = true;
                    Form1.Variables.StarTrekMode = false;
                    Form1.Variables.CometTails = false;
                    Form1.Variables.CustomStarColor = false;
                    Form1.Variables.StarCount = 300;
                    Form1.Variables.StarSizeIncreasePerStep = 0.004;
                    Form1.Variables.StarAlphaIncreasePerStep = 2;
                    Form1.Variables.StarStartingAlpha = 50;
                    Form1.Variables.SpeedFactor = 2;
                    Form1.Variables.InitialStarSize = 2;
                    Form1.Variables.ColorfulStars = false;
                    Form1.Variables.PictureBoxBackgroundColorR = 0;
                    Form1.Variables.PictureBoxBackgroundColorG = 0;
                    Form1.Variables.PictureBoxBackgroundColorB = 0;
                    PictureBoxFromForm1.BackColor = Color.FromArgb(255, Form1.Variables.PictureBoxBackgroundColorR, Form1.Variables.PictureBoxBackgroundColorG, Form1.Variables.PictureBoxBackgroundColorB);
                    SetButtons();
                    SetSliders();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonStarTrekMode_Click(object sender, EventArgs e)
        {
            try
            {
                if (!Form1.Variables.StarTrekMode)
                {
                    Form1.Variables.DefaultMode = false;
                    Form1.Variables.PartyMode = false;
                    Form1.Variables.ClassicMode = false;
                    Form1.Variables.StarTrekMode = true;
                    Form1.Variables.CustomStarColor = false;
                    Form1.Variables.StarCount = 300;
                    Form1.Variables.StarSizeIncreasePerStep = 0.00001;
                    Form1.Variables.StarAlphaIncreasePerStep = 0.5;
                    Form1.Variables.StarStartingAlpha = 50;
                    Form1.Variables.SpeedFactor = 1;
                    Form1.Variables.InitialStarSize = 0.1;
                    Form1.Variables.ColorfulStars = false;
                    Form1.Variables.CometTails = true;
                    Form1.Variables.CometTailLength = 0.2;
                    Form1.Variables.CometTailWidth = 3;
                    Form1.Variables.PictureBoxBackgroundColorR = 0;
                    Form1.Variables.PictureBoxBackgroundColorG = 0;
                    Form1.Variables.PictureBoxBackgroundColorB = 0;
                    PictureBoxFromForm1.BackColor = Color.FromArgb(255, Form1.Variables.PictureBoxBackgroundColorR, Form1.Variables.PictureBoxBackgroundColorG, Form1.Variables.PictureBoxBackgroundColorB);
                    SetButtons();
                    SetSliders();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void TrackBarStars_Scroll_1(object sender, EventArgs e)
        {
            try
            {
                // Take the trackbar value and scale it logarithmicly, assign the result to variable
                Form1.Variables.StarCount = Convert.ToInt32(ScaleLogarithmicSlider(TrackBarStars.Value, 300, 30000));
                labelStars.Text = Form1.Variables.StarCount.ToString();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void TrackBarVelocity_Scroll_1(object sender, EventArgs e)
        {
            try
            {
                // Take the trackbar value and scale it logarithmicly, assign the result to variable
                Form1.Variables.SpeedFactor = ScaleLogarithmicSlider(TrackBarVelocity.Value, 0.05, 32);
                labelVelocity.Text = (TrackBarVelocity.Value + 1).ToString();

                if (Form1.Variables.SyncParameters)
                {
                    SyncParameters(Form1.Variables.SpeedFactor, 0, 0);
                    SetSliders();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void TrackBarStarSize_Scroll_1(object sender, EventArgs e)
        {
            try
            {
                // Take the trackbar value and scale it logarithmicly, assign the result to variable
                Form1.Variables.InitialStarSize = ScaleLogarithmicSlider(TrackBarStarSize.Value, 0.05, 32);
                labelStarSize.Text = (TrackBarStarSize.Value + 1).ToString();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void TrackBarStarSizeIncreaseRate_Scroll_1(object sender, EventArgs e)
        {
            try
            {
                // Take the trackbar value and scale it logarithmicly, assign the result to variable
                Form1.Variables.StarSizeIncreasePerStep = ScaleLogarithmicSlider(TrackBarStarSizeIncreaseRate.Value, 0.00001, 0.1);
                labelStarSizeIncreaseRate.Text = (TrackBarStarSizeIncreaseRate.Value + 1).ToString();

                if (Form1.Variables.SyncParameters)
                {
                    SyncParameters(0, 0, Form1.Variables.StarSizeIncreasePerStep);
                    SetSliders();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void TrackBarBrightnessIncreaseRate_Scroll_1(object sender, EventArgs e)
        {
            try
            {
                // Take the trackbar value and scale it logarithmicly, assign the result to variable
                Form1.Variables.StarAlphaIncreasePerStep = ScaleLogarithmicSlider(TrackBarBrightnessIncreaseRate.Value, 0.01, 8);
                labelBrightnessIncreaseRate.Text = (TrackBarBrightnessIncreaseRate.Value + 1).ToString();

                if (Form1.Variables.SyncParameters)
                {
                    SyncParameters(0, Form1.Variables.StarAlphaIncreasePerStep, 0);
                    SetSliders();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void TrackBarCometTailLength_Scroll_1(object sender, EventArgs e)
        {
            try
            {
                // Take the trackbar value and scale it logarithmicly, assign the result to variable
                Form1.Variables.CometTailLength = ScaleLogarithmicSlider(TrackBarCometTailLength.Value, 0.1, 1);
                labelCometTailLength.Text = Convert.ToInt32(TrackBarCometTailLength.Value + 1).ToString();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void TrackBarCometTailWidth_Scroll(object sender, EventArgs e)
        {
            try
            {
                // Take the trackbar value and scale it logarithmicly, assign the result to variable
                Form1.Variables.CometTailWidth = TrackBarCometTailWidth.Value;
                labelCometTailWidth.Text = (TrackBarCometTailWidth.Value + 1).ToString();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void TrackBarMasterClockFrequency_Scroll(object sender, EventArgs e)
        {
            try
            {
                // Scale 0-10 trackbar value to 1-11
                int scaled = TrackBarMasterClockFrequency.Value + 1;
                // Invert the trackbar value to get the timer interval
                // InvertedInterval = (max + min) - num;
                int InvertedInterval = (11 + 1) - scaled;
                // Do a little scaleing
                InvertedInterval = InvertedInterval * 10;
                Form1.Variables.TimerInterval = InvertedInterval;
                this.TimerFromForm1.Stop();
                this.TimerFromForm1.Interval = Form1.Variables.TimerInterval;
                this.TimerFromForm1.Start();
                LabelMasterClockFrequency.Text = (Form1.Variables.TimerInterval).ToString() + " ms";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonAbout_Click(object sender, EventArgs e)
        {
            FormAbout frm = new FormAbout();
            frm.TopMost = true;
            frm.Show();
        }

        public void ButtonSetBackgroundColor_Click(object sender, EventArgs e)
        {
            try
            {
                //Color Dialog  
                ColorDialog MyColorDialog = new ColorDialog();
                DialogResult MyColorDialogResult = MyColorDialog.ShowDialog();
                if (MyColorDialogResult == DialogResult.OK)
                {
                    PictureBoxFromForm1.BackColor = MyColorDialog.Color;
                    Form1.Variables.PictureBoxBackgroundColorR = MyColorDialog.Color.R;
                    Form1.Variables.PictureBoxBackgroundColorG = MyColorDialog.Color.G;
                    Form1.Variables.PictureBoxBackgroundColorB = MyColorDialog.Color.B;
                    PictureBoxFromForm1.BackColor = Color.FromArgb(255, Form1.Variables.PictureBoxBackgroundColorR, Form1.Variables.PictureBoxBackgroundColorG, Form1.Variables.PictureBoxBackgroundColorB);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonSetStarColor_Click(object sender, EventArgs e)
        {
            try
            {
                //Color Dialog  
                ColorDialog MyColorDialog = new ColorDialog();
                DialogResult MyColorDialogResult = MyColorDialog.ShowDialog();

                if (MyColorDialogResult == DialogResult.OK)
                {
                    // Get RGB values from custom color
                    Form1.Variables.CustomStarColorR = MyColorDialog.Color.R;
                    Form1.Variables.CustomStarColorG = MyColorDialog.Color.G;
                    Form1.Variables.CustomStarColorB = MyColorDialog.Color.B;
                    Form1.Variables.CustomStarColor = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonExit_Click(object sender, EventArgs e)
        {
            try
            {
                Form1.WriteVariables("ultra_starfield_settings.xml");
                System.Windows.Forms.Application.Exit();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog SaveFileDialog1 = new SaveFileDialog();
                SaveFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();
                SaveFileDialog1.Filter = "xml files (*.xml)|*.xml|All files (*.*)|*.*";
                SaveFileDialog1.FilterIndex = 0;
                SaveFileDialog1.RestoreDirectory = true;

                if (SaveFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    string selectedFileName = SaveFileDialog1.FileName;
                    Form1.WriteVariables(selectedFileName);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonLoad_Click(object sender, EventArgs e)
        {
            try
            {
                OpenFileDialog OpenFileDialog1 = new OpenFileDialog();
                OpenFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();
                OpenFileDialog1.Filter = "xml files (*.xml)|*.xml|All files (*.*)|*.*";
                OpenFileDialog1.FilterIndex = 0;
                OpenFileDialog1.RestoreDirectory = true;

                if (OpenFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    string selectedFileName = OpenFileDialog1.FileName;
                    Form1.ReadVariables(selectedFileName);
                }
                SetSliders();
                SetButtons(); // needed?
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonHelp_Click(object sender, EventArgs e)
        {
            FormHelp frm = new FormHelp();
            frm.TopMost = true;
            frm.Show();
        }

        private void ButtonSilenceErrors_Click(object sender, EventArgs e)
        {
            try
            {
                if (Form1.Variables.SilenceErrors)
                {
                    Form1.Variables.SilenceErrors = false;
                }
                else
                {
                    Form1.Variables.SilenceErrors = true;
                }
                SetButtons();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void frmOptions_FormClosed(object sender, FormClosedEventArgs e)
        {
            Cursor.Hide();
        }
    }
}